@@ -2,7 +2,7 @@
 package MRTG_lib;
 
 ###################################################################
-# MRTG 2.9.25  Support library MRTG_lib.pm
+# MRTG 2.9.29  Support library MRTG_lib.pm
 ###################################################################
 # Created by Tobias Oetiker <oetiker@ee.ethz.ch>
 #            and Dave Rand <dlr@bungi.com>
@@ -17,7 +17,7 @@ package MRTG_lib;
 
 require 5.005;
 use strict;
-use SNMP_util "0.86";
+use SNMP_util "0.93";
 use vars qw($OS $SL $PS @EXPORT @ISA $VERSION %mrtgrules);
 
 BEGIN {
@@ -41,10 +41,10 @@ require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw(readcfg cfgcheck setup_loghandlers
 	     datestr expistr ensureSL timestamp
-             demonize_me debug log2rrd storeincache
+             create_pid demonize_me debug log2rrd storeincache
 	     populateconfcache readconfcache writeconfcache);
 
-$VERSION = 2.090017;
+$VERSION = 2.090026;
 
 
 %mrtgrules =
@@ -161,16 +161,16 @@ $VERSION = 2.090017;
        [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
 
        'unscaled[]' => 
-       [sub{$_[0] =~ /[dwmy]+/i}, sub{"Must be a string of [d]ay, [w]eek, [m]onth, [y]ear"}],
+       [sub{$_[0] =~ /[ndwmy]+/i}, sub{"Must be a string of [n]one, [d]ay, [w]eek, [m]onth, [y]ear"}],
 
        'weekformat[]' => 
        [sub{$_[0] =~ /[UVW]/}, sub{"Must be either W, V, or U"}],
 
        'withpeak[]' =>
-       [sub{$_[0] =~ /[dwmy]+/i}, sub{"Must be a string of [d]ay, [w]eek, [m]onth, [y]ear"}],
+       [sub{$_[0] =~ /[ndwmy]+/i}, sub{"Must be a string of [n]one, [d]ay, [w]eek, [m]onth, [y]ear"}],
 
        'suppress[]' =>
-       [sub{$_[0] =~ /[dwmy]+/i}, sub{"Must be a string of [d]ay, [w]eek, [m]onth, [y]ear"}],
+       [sub{$_[0] =~ /[ndwmy]+/i}, sub{"Must be a string of [n]one, [d]ay, [w]eek, [m]onth, [y]ear"}],
 
        'xsize[]' =>
        [sub{((int($_[0]) >= 30) && (int($_[0]) <= 600))}, sub{"$_[0] must be between 30 and 600 pixels"}],
@@ -397,6 +397,9 @@ sub readcfg ($$$$;$$) {
         if (/^([A-Za-z0-9*]+)\[(\S+)\]\s*:\s*(.*\S?)\s*$/) {
             $first = lc($1);
             $second = lc($2);
+            # For us spelling-handicapped Americans. ;)
+            # James Overbeck, grendel@gmo.jp, 2003/01/19
+            if ($first eq 'colors') { $first = 'colours' };
             if ($second eq '^') {
                 if ($3 ne '') {
                     $pre{$first}=$3;
@@ -637,9 +640,10 @@ sub cfgcheck ($$$$) {
                 unless defined $found;
     }
     if (defined $$cfg{snmpoptions}) {
-           $$cfg{snmpoptions} = eval('{ avoid_negative_request_ids => 1, '.$$cfg{snmpoptions}.'}');
+           $cfg->{snmpoptions} = eval('{'.$cfg->{snmpoptions}.'}');
+           $cfg->{snmpoptions}{avoid_negative_request_ids} = 1;
     } else {
-           $$cfg{snmpoptions} = {avoid_negative_request_ids => 1};
+           $cfg->{snmpoptions} = {avoid_negative_request_ids => 1};
     }
 
     # default interval is 5 minutes
@@ -916,22 +920,33 @@ sub expistr ($) {
     return "$wday, $mday $month ".($year+1900)." $hour:$min:$sec GMT";
 }
 
+sub create_pid ($) {
+    my $pidfile = shift;
+    return if $OS eq 'NT';
+    return if -e $pidfile;
+    if ( open(PIDFILE,">$pidfile")) {
+         close PIDFILE;
+    } else {
+         warn "cannot write to $pidfile: $!\n";
+    }
+}
+
 sub demonize_me ($) {
     my $pidfile = shift;
     print "Daemonizing MRTG ...\n";
     if ( $OS eq 'NT' ) {
-	print "Do Not close this window. Or MRTG will die\n";
+        print "Do Not close this window. Or MRTG will die\n";
 #            require Win32::Console;
 #            my $CONSOLE = new Win32::Console;
-	#    detach process from Console
+        #    detach process from Console
 #            $CONSOLE->Flush();
 #            $CONSOLE->Free();
-#	    $CONSOLE->Alloc();
-#	    $CONSOLE->Mode()
+#            $CONSOLE->Alloc();
+#            $CONSOLE->Mode()
     } else {
            # Check out if there is another mrtg running before forking
            if (defined $pidfile && open(READPID, "<$pidfile")) {
-               chomp(my $input = <READPID>);	# read process id in pidfile
+               chomp(my $input = <READPID>);    # read process id in pidfile
                if ($input && kill 0 => $input) {# oops - the pid actually exists
                    die "ERROR: I Quit! Another copy of mrtg seems to be running. Check $pidfile\n";
                }
@@ -942,20 +957,19 @@ sub demonize_me ($) {
            if ($pid) {
               exit;
             } else {
-                if (defined $pidfile){ 
-                   $main::Cleanfile3 = $pidfile;   
+                if (defined $pidfile){
+                   $main::Cleanfile3 = $pidfile;
                    if (open(PIDFILE,">$pidfile")) {
                         print PIDFILE "$$\n";
                         close PIDFILE;
                    } else {
                         warn "cannot write to $pidfile: $!\n";
                    }
-              }                 
+              }
               require 'POSIX.pm';
               &POSIX::setsid or die "Can't start a new session: $!";
               open STDOUT,'>/dev/null' or die "ERROR: Redirecting STDOUT to /dev/null: $!";
               open STDIN, '</dev/null' or die "ERROR: Redirecting STDIN from /dev/null: $!";
-                  
       }
    }
 }
@@ -1175,6 +1189,7 @@ sub readconfcache ($) {
             next unless /\t/; #ignore odd lines
             my ($host,$method,$key,$if) = split (/\t/, $_);
             $key =~ s/[\0- ]+$//; # no trailing whitespace in keys realy !
+            $key =~ s/[\0- ]/ /g; # all else becomes a normal space ... get a life
             $confcache{$host}{$method}{$key} = $if;
         }
         close CFGOK;
@@ -1213,6 +1228,8 @@ sub writeconfcache ($$) {
 
 sub storeincache ($$$$$){
     my($confcache,$host,$method,$key,$value) = @_;
+    $value =~ s/[\0- ]/ /g; # all else becomes a normal space ... get a life
+    $value =~ s/ +$//; # no trailing spaces
     if (defined $$confcache{$host}{$method}{$key} and 
 	$$confcache{$host}{$method}{$key} ne $value) {
         $$confcache{$host}{$method}{$key} = "Dup";
@@ -1254,6 +1271,7 @@ sub populateconfcache ($$$$) {
 		      storeincache($confcache,$host,$tables{$node},$oid,$desc);
 		  } else {
                       $desc =~ s/[\0- ]+$//; #trailing whitespace is too sick for us
+                      $desc =~ s/[\0- ]/ /g; #whitespace is just whitespace
 		      storeincache($confcache,$host,$tables{$node},$desc,$oid);
 		  }
 	      };
@@ -1745,6 +1763,12 @@ C<expistr(time)>
 Returns the time given in the argument formatted suitable for HTTP
 Expire-Headers.
 
+=item C<create_pid> 
+
+C<create_pid()> 
+
+Creates a pid file for the mrtg daemon       
+
 =item C<demonize_me>
 
 C<demonize_me()>
