@@ -1,5 +1,5 @@
 /*
- MRTG 2.9.27  -- Rateup
+ MRTG 2.9.29  -- Rateup
  *********************
 
  Rateup is a fast add-on to the great MRTG Traffic monitor.  It makes
@@ -34,7 +34,15 @@
 #include <string.h>
 /* VC++ does not have unistd.h */
 #ifndef WIN32
-#include <unistd.h>
+  #include "../config.h"
+  /* maybe this will make rateup.c compile on HPUX */
+  #if !defined(HAVE_STRTOLL) && defined(HAVE___STRTOLL)
+     #ifdef HAVE_INTTYPES_H
+       #include <inttypes.h>
+     #endif
+     #define strtoll __strtoll
+  #endif
+  #include <unistd.h>
 #endif
 #include <limits.h>
 #include <stdio.h>
@@ -69,7 +77,7 @@
 #include <gd.h>
 #include <gdfonts.h>
 
-char *VERSION = "2.9.27";
+char *VERSION = "2.9.29";
 char *program,*router,*routerpath;
 int histvalid;
 
@@ -272,7 +280,7 @@ int peak;
     maxvi *= muli;
     maxvo *= muli;
     origmaxvi = maxvi < (long long)0 ? -maxvi : maxvi;
-    origmaxvo = maxvo < (long long)0 ? maxvo : maxvo;
+    origmaxvo = maxvo < (long long)0 ? -maxvo : maxvo;
     maxv = (long long) max(maxvi,maxvo);
     if (step > MONTH_SAMPLE) {
       type = 4;
